"""create initial tables

Revision ID: 6d5ae5b9c541
Revises: 
Create Date: 2025-05-07 19:10:39.137004

"""
from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision: str = '6d5ae5b9c541'
down_revision: Union[str, None] = None
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    """Upgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('log_flows',
    sa.Column('id', sa.UUID(), nullable=False),
    sa.Column('transaction_id', sa.UUID(), nullable=False),
    sa.Column('flow_name', sa.String(length=255), nullable=False),
    sa.Column('flow_author', sa.Enum('user', 'system', name='logflowauthor'), nullable=False),
    sa.Column('started_at', sa.DateTime(timezone=True), nullable=False),
    sa.Column('ended_at', sa.DateTime(timezone=True), nullable=True),
    sa.Column('status', sa.Enum('in_progress', 'success', 'failure', name='logstepstatus'), nullable=True),
    sa.Column('s3_key', sa.Text(), nullable=True),
    sa.PrimaryKeyConstraint('id'),
    sa.UniqueConstraint('transaction_id')
    )
    op.create_table('log_steps',
    sa.Column('id', sa.UUID(), nullable=False),
    sa.Column('flow_id', sa.UUID(), nullable=False),
    sa.Column('step_name', sa.String(length=255), nullable=False),
    sa.Column('status', sa.Enum('in_progress', 'success', 'failure', name='logstepstatus', native_enum=False), nullable=False),
    sa.Column('start_time', sa.DateTime(timezone=True), nullable=False),
    sa.Column('end_time', sa.DateTime(timezone=True), nullable=True),
    sa.Column('duration_ms', sa.Integer(), nullable=True),
    sa.Column('error_message', sa.String(), nullable=True),
    sa.Column('step_metadata', sa.JSON(), nullable=True),
    sa.ForeignKeyConstraint(['flow_id'], ['log_flows.id'], ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('id')
    )
    op.create_table('log_step_exceptions',
    sa.Column('id', sa.UUID(), nullable=False),
    sa.Column('step_id', sa.UUID(), nullable=False),
    sa.Column('type', sa.String(length=128), nullable=False),
    sa.Column('message', sa.String(), nullable=True),
    sa.Column('traceback', sa.String(), nullable=True),
    sa.Column('created_at', sa.DateTime(timezone=True), nullable=False),
    sa.ForeignKeyConstraint(['step_id'], ['log_steps.id'], ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('id')
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    """Downgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('log_step_exceptions')
    op.drop_table('log_steps')
    op.drop_table('log_flows')
    # ### end Alembic commands ###



import uuid
from dataclasses import asdict
from typing import List, Generator, Union

from requests import exceptions as requests_exceptions

from omnisdk.omnitron.models import (
    Customer,
    Address,
    CargoCompany,
    Order,
    BatchRequest,
    CancellationRequest,
)
from omnisdk.omnitron.endpoints import ChannelIntegrationActionEndpoint

from channel_app.core import settings
from channel_app.core.data import (
    BatchRequestResponseDto,
    CancellationRequestDto,
    ChannelCancellationRequestDto,
    OmnitronCreateOrderDto,
    OmnitronOrderDto,
    ChannelCreateOrderDto,
    ErrorReportDto,
    OrderBatchRequestResponseDto,
    CancelOrderDto,
    ChannelUpdateOrderItemDto,
)
from channel_app.core.settings import OmnitronIntegration, ChannelIntegration
from channel_app.logs.services import LogService
from channel_app.omnitron.batch_request import ClientBatchRequest
from channel_app.omnitron.constants import (
    BatchRequestStatus,
    ContentType,
    FailedReasonType,
)
from channel_app.omnitron.exceptions import (
    CityException,
    TownshipException,
    DistrictException,
    CargoCompanyException,
    OrderException,
)


class OrderService(object):
    batch_service = ClientBatchRequest

    def fetch_and_create_order(self, is_success_log=True):
        log_service = LogService()
        tx_id = uuid.uuid4()
        log_service.create_flow(
            name="OrderSync",
            transaction_id=tx_id,
        )

        try:
            with log_service.step("fetch_orders"):
                with OmnitronIntegration(
                    content_type=ContentType.order.value
                ) as omnitron_integration:

                    with log_service.step("get_orders"):
                        get_orders = ChannelIntegration().do_action(
                            key="get_orders",
                            batch_request=omnitron_integration.batch_request,
                        )

                    get_orders: Generator
                    order_batch_objects = []
                    while True:
                        try:
                            channel_create_order, report_list, _ = next(get_orders)
                        except StopIteration:
                            break

                        # tips
                        channel_create_order: ChannelCreateOrderDto
                        metadata = {"order_number": channel_create_order.order.number}

                        report_list: List[ErrorReportDto]
                        for report in report_list:
                            if is_success_log or not report.is_ok:
                                report.error_code = (
                                    f"{omnitron_integration.batch_request.local_batch_id}"
                                    f"-Channel-GetOrders_{channel_create_order.order.number}"
                                )
                                try:

                                    with log_service.step(
                                        "create_error_report", metadata=metadata
                                    ):
                                        omnitron_integration.do_action(
                                            key="create_error_report", objects=report
                                        )
                                except Exception as err:
                                    log_service.add_exception(err)
                                    raise

                        try:
                            with log_service.step("create_order", metadata=metadata):
                                order = self.create_order(
                                    omnitron_integration=omnitron_integration,
                                    channel_order=channel_create_order,
                                )
                        except Exception as err:
                            log_service.add_exception(err)
                            raise

                        if order and omnitron_integration.batch_request.objects:
                            order_batch_objects.extend(
                                omnitron_integration.batch_request.objects
                            )

                    omnitron_integration.batch_request.objects = order_batch_objects

                    with log_service.step("batch_to_done"):
                        try:
                            self.batch_service(settings.OMNITRON_CHANNEL_ID).to_done(
                                batch_request=omnitron_integration.batch_request
                            )
                        except requests_exceptions.HTTPError as exc:
                            log_service.add_exception(exc)
                            if (
                                exc.response.status_code == 406
                                and "batch_request_status_100_1" in exc.response.text
                            ):
                                pass
                            else:
                                raise exc
        except Exception as fatal:
            log_service.add_exception(fatal)
            raise
        finally:
            log_service.save()

    def create_order(
        self,
        omnitron_integration: OmnitronIntegration,
        channel_order: ChannelCreateOrderDto,
    ) -> Union[Order, None]:
        log_service = LogService()
        tx_id = uuid.uuid4()
        log_service.create_flow(name="Create Order", transaction_id=tx_id)

        try:
            order = channel_order.order

            with log_service.step("get_or_create_customer"):
                try:
                    customer = omnitron_integration.do_action(
                        key="get_or_create_customer", objects=order.customer
                    )[0]
                except Exception as exc:
                    log_service.add_exception(exc)
                    return

            customer: Customer
            with log_service.step("combine_addresses"):
                try:
                    shipping_address_data = {
                        "customer": customer,
                        "address": order.shipping_address,
                    }

                    with log_service.step("get_or_create_address (shipping)"):
                        shipping_address = omnitron_integration.do_action(
                            key="get_or_create_address", objects=shipping_address_data
                        )[0]

                    billing_address_data = {
                        "customer": customer,
                        "address": order.billing_address,
                    }

                    with log_service.step("get_or_create_address (billing)"):
                        billing_address = omnitron_integration.do_action(
                            key="get_or_create_address", objects=billing_address_data
                        )[0]
                except (CityException, TownshipException, DistrictException) as exc:
                    log_service.add_exception(exc)

                    with log_service.step("create_address_error_report"):
                        omnitron_integration.do_action(
                            key="create_address_error_report", object=exc
                        )
                    return

                except IndexError as exc:
                    log_service.add_exception(exc)
                    return

            shipping_address: Address
            billing_address: Address

            with log_service.step("get_cargo_company"):
                try:
                    cargo_company = omnitron_integration.do_action(
                        key="get_cargo_company", objects=order.cargo_company
                    )[0]
                except (CargoCompanyException, IndexError) as exc:
                    log_service.add_exception(exc)
                    return

            cargo_company: CargoCompany

            order_data = asdict(order)
            order_data["customer"] = customer.pk
            order_data["shipping_address"] = shipping_address.pk
            order_data["billing_address"] = billing_address.pk
            order_data["cargo_company"] = cargo_company.pk

            with log_service.step("create_order_dto"):
                omnitron_order = OmnitronOrderDto(**order_data)

            with log_service.step("create_omnitron_order_dto"):
                create_order_dto = OmnitronCreateOrderDto(
                    order=omnitron_order, order_item=channel_order.order_item
                )

            with log_service.step("create_order"):
                try:
                    orders: List[Order] = omnitron_integration.do_action(
                        key="create_order", objects=create_order_dto
                    )
                    order = orders[0]
                except (OrderException, IndexError) as exc:
                    log_service.add_exception(exc)
                    return

            return order
        except Exception as exc:
            log_service.add_exception(exc)
            raise
        finally:
            log_service.save()

    def fetch_and_update_order_items(self, is_success_log=True):
        log_service = LogService()
        tx_id = uuid.uuid4()
        log_service.create_flow(
            name="Fetch and Update Order Items", transaction_id=tx_id
        )

        try:
            with log_service.step("fetch_and_update_order_items"):
                with OmnitronIntegration(
                    content_type=ContentType.order.value
                ) as omnitron_integration:

                    with log_service.step("get_updated_order_items"):
                        get_updated_orders = ChannelIntegration().do_action(
                            key="get_updated_order_items",
                            batch_request=omnitron_integration.batch_request,
                        )
                    get_updated_orders: Generator
                    order_batch_objects = []
                    while True:
                        try:
                            channel_update_order, report_list, _ = next(
                                get_updated_orders
                            )
                        except StopIteration:
                            break

                        # tips
                        channel_update_order: ChannelUpdateOrderItemDto
                        report_list: List[ErrorReportDto]
                        for report in report_list:
                            if report and (is_success_log or not report.is_ok):
                                report.error_code = (
                                    f"{omnitron_integration.batch_request.local_batch_id}"
                                    f"_GetUpdatedOrders_{channel_update_order.remote_id}"
                                )

                                with log_service.step(
                                    "create_error_report",
                                    metadata=channel_update_order.order_number,
                                ):
                                    omnitron_integration.do_action(
                                        key="create_error_report", objects=report
                                    )

                        with log_service.step("update_order_items"):
                            omnitron_integration.do_action(
                                key="update_order_items", objects=channel_update_order
                            )

                    omnitron_integration.batch_request.objects = order_batch_objects

                    with log_service.step("batch_to_done"):
                        self.batch_service(settings.OMNITRON_CHANNEL_ID).to_done(
                            batch_request=omnitron_integration.batch_request
                        )
        except Exception as fatal:
            log_service.add_exception(fatal)
            raise
        finally:
            log_service.save()

    def update_orders(self, is_sync=True, is_success_log=True, add_order_items=False):
        log_service = LogService()
        tx_id = uuid.uuid4()
        log_service.create_flow(name="Update Orders", transaction_id=tx_id)

        try:
            with log_service.step("update_orders"):
                with OmnitronIntegration(
                    content_type=ContentType.order.value
                ) as omnitron_integration:

                    with log_service.step("get_orders"):
                        orders = omnitron_integration.do_action(key="get_orders")

                    orders: List[Order]
                    first_order_count = len(orders)
                    if add_order_items:
                        with log_service.step("get_order_items_with_order"):
                            orders = orders and omnitron_integration.do_action(
                                key="get_order_items_with_order", objects=orders
                            )

                    if not orders:
                        if first_order_count:
                            omnitron_integration.batch_request.objects = None

                            with log_service.step("batch_to_fail"):
                                self.batch_service(
                                    omnitron_integration.channel_id
                                ).to_fail(omnitron_integration.batch_request)
                        return

                    with log_service.step("send_updated_orders"):
                        response_data, reports, data = ChannelIntegration().do_action(
                            key="send_updated_orders",
                            objects=orders,
                            batch_request=omnitron_integration.batch_request,
                            is_sync=True,
                        )

                    # tips
                    response_data: List[OrderBatchRequestResponseDto]
                    reports: List[ErrorReportDto]
                    data: List[Order]

                    if not is_sync:
                        if reports[0].is_ok:
                            with log_service.step("batch_send_to_remote"):
                                self.batch_service(
                                    settings.OMNITRON_CHANNEL_ID
                                ).to_sent_to_remote(
                                    batch_request=omnitron_integration.batch_request
                                )
                        else:
                            is_sync = True

                    if reports and (is_success_log or not reports[0].is_ok):
                        for report in reports:
                            with log_service.step("create_error_report"):
                                omnitron_integration.do_action(
                                    key="create_error_report", objects=report
                                )

                    if is_sync:
                        with log_service.step("process_order_batch_requests"):
                            omnitron_integration.do_action(
                                key="process_order_batch_requests",
                                objects=response_data,
                            )
        except Exception as e:
            log_service.add_exception(e)
            raise
        finally:
            log_service.save()

    def get_order_batch_requests(self, is_success_log=False):
        log_service = LogService()
        tx_id = uuid.uuid4()
        log_service.create_flow(name="Get Order Batch Requests", transaction_id=tx_id)

        try:
            with log_service.step("get_order_batch_requests"):
                with OmnitronIntegration(create_batch=False) as omnitron_integration:
                    with log_service.step("get_batch_requests"):
                        batch_request_data = omnitron_integration.do_action(
                            "get_batch_requests",
                            params={
                                "status": ["sent_to_remote", "ongoing"],
                                "content_type": ContentType.order.value,
                            },
                        )

                    # tips
                    batch_request_data: List[BatchRequest]

                    for batch_request in batch_request_data:
                        with log_service.step("check_orders"):
                            (
                                response_data,
                                report,
                                data,
                            ) = ChannelIntegration().do_action(
                                key="check_orders", objects=batch_request
                            )

                        # tips
                        response_data: List[OrderBatchRequestResponseDto]
                        report: ErrorReportDto
                        data: BatchRequest

                        if report and (is_success_log or not report.is_ok):
                            with log_service.step("create_error_report"):
                                omnitron_integration.do_action(
                                    key="create_error_report", objects=report
                                )

                        if response_data:
                            omnitron_integration.batch_request = batch_request_data

                            with log_service.step("process_order_batch_requests"):
                                omnitron_integration.do_action(
                                    key="process_order_batch_requests",
                                    objects=response_data,
                                )
        except Exception as fatal:
            log_service.add_exception(fatal)
            raise
        finally:
            log_service.save()

    def fetch_and_create_cancel(self, is_success_log=True):
        log_service = LogService()
        tx_id = uuid.uuid4()
        log_service.create_flow(name="Fetch and Create Cancel", transaction_id=tx_id)

        try:
            with log_service.step("fetch_and_create_cancel"):
                with OmnitronIntegration(
                    content_type=ContentType.order.value
                ) as omnitron_integration:
                    with log_service.step("get_cancelled_orders"):
                        get_cancelled_order = ChannelIntegration().do_action(
                            key="get_cancelled_orders",
                            batch_request=omnitron_integration.batch_request,
                        )

                    get_cancelled_order: Generator

                    while True:
                        try:
                            cancel_order_dto, report, _ = next(get_cancelled_order)
                        except StopIteration:
                            break

                        # tips
                        cancel_order_dto: CancelOrderDto
                        report: ErrorReportDto

                        if report and (is_success_log or not report.is_ok):
                            with log_service.step("create_error_report"):
                                omnitron_integration.do_action(
                                    key="create_error_report", objects=report
                                )

                        with log_service.step("create_cancel"):
                            self.create_cancel(
                                omnitron_integration=omnitron_integration,
                                cancel_order_dto=cancel_order_dto,
                            )

        except Exception as fatal:
            log_service.add_exception(fatal)
            raise
        finally:
            log_service.save()

    def create_cancel(
        self,
        omnitron_integration: OmnitronIntegration,
        cancel_order_dto: CancelOrderDto,
    ):
        success_datas: List[Order] = omnitron_integration.do_action(
            key="create_order_cancel", objects=cancel_order_dto
        )
        try:
            success_data = success_datas[0]
            return success_data
        except IndexError:
            return

    def fetch_and_create_cancellation_requests(self, is_success_log=True):
        log_service = LogService()
        tx_id = uuid.uuid4()
        log_service.create_flow(
            name="Fetch and Create Cancellation Requests", transaction_id=tx_id
        )

        try:
            with log_service.step("fetch_and_create_cancellation_requests"):
                with OmnitronIntegration(
                    content_type=ContentType.cancellation_request.value
                ) as omnitron_integration:
                    with log_service.step("get_cancellation_requests"):
                        get_cancellation_requests = ChannelIntegration().do_action(
                            key="get_cancellation_requests",
                            batch_request=omnitron_integration.batch_request,
                        )

                    get_cancellation_requests: Generator
                    while True:
                        try:
                            cancellation_request, report_list, _ = next(
                                get_cancellation_requests
                            )
                        except StopIteration:
                            break

                        # tips
                        cancellation_request: CancellationRequestDto
                        report_list: List[ErrorReportDto]
                        for report in report_list:
                            if report and (is_success_log or not report.is_ok):
                                report.error_code = (
                                    f"{omnitron_integration.batch_request.local_batch_id}"
                                    f"-Channel-GetCancellationRequests_{cancellation_request.order_item}"
                                )

                                with log_service.step("create_error_report"):
                                    omnitron_integration.do_action(
                                        key="create_error_report", objects=report
                                    )

                        # omnitron integration do action create_cancellation_request
                        with log_service.step("create_cancellation_requests"):
                            cancellation_request_response = (
                                omnitron_integration.do_action(
                                    key="create_cancellation_requests",
                                    objects=cancellation_request,
                                )
                            )
                    try:
                        with log_service.step("batch_to_done"):
                            self.batch_service(settings.OMNITRON_CHANNEL_ID).to_done(
                                batch_request=omnitron_integration.batch_request
                            )
                    except requests_exceptions.HTTPError as exc:
                        log_service.add_exception(exc)
                        if (
                            exc.response.status_code == 406
                            and "batch_request_status_100_1" in exc.response.text
                        ):
                            pass
                        else:
                            raise exc
        except Exception as fatal:
            log_service.add_exception(fatal)
            raise
        finally:
            log_service.save()

    def update_cancellation_requests(self, is_success_log=True):
        log_service = LogService()
        log_service.create_flow(name="Update Cancellation Requests")

        try:
            with log_service.step("update_cancellation_requests"):
                with OmnitronIntegration(
                    content_type=ContentType.cancellation_request.value
                ) as omnitron_integration:
                    with log_service.step("get_cancellation_requests_update"):
                        cancellation_requests = omnitron_integration.do_action(
                            key="get_cancellation_requests_update", objects={}
                        )

                    cancellation_requests: List[CancellationRequest]

                    if not cancellation_requests:
                        omnitron_integration.batch_request.objects = None
                        with log_service.step("batch_to_fail"):
                            self.batch_service(omnitron_integration.channel_id).to_fail(
                                omnitron_integration.batch_request
                            )

                        return

                    batch_request_object_list = []

                    for cancellation_request in cancellation_requests:
                        with log_service.step("get_channel_order_item"):
                            remote_order_item = self.get_channel_order_item(
                                omnitron_integration, cancellation_request.order_item
                            )

                        with log_service.step("get_channel_reason"):
                            remote_reason = self.get_channel_reason(
                                omnitron_integration, cancellation_request.reason
                            )

                        with log_service.step("get_channel_cancellation_request"):
                            remote_cancellation_request = (
                                self.get_channel_cancellation_request(
                                    omnitron_integration, cancellation_request.id
                                )
                            )

                        with log_service.step(
                            "create_channel_cancellation_request_dto"
                        ):
                            channel_cancellation_request = ChannelCancellationRequestDto(
                                cancellation_type=cancellation_request.cancellation_type,
                                status=cancellation_request.status,
                                order_item=remote_order_item,
                                reason=remote_reason,
                                description=cancellation_request.description,
                                remote_id=remote_cancellation_request,
                            )

                        with log_service.step(
                            "update_cancellation_request",
                            metadata=channel_cancellation_request.remote_id,
                        ):
                            (
                                response_data,
                                reports,
                                data,
                            ) = ChannelIntegration().do_action(
                                key="update_cancellation_request",
                                objects=channel_cancellation_request,
                                batch_request=omnitron_integration.batch_request,
                                is_sync=True,
                            )

                        # tips
                        response_data: List[BatchRequestResponseDto]
                        reports: List[ErrorReportDto]
                        data: List[ChannelCancellationRequestDto]

                        if reports and (is_success_log or not reports[0].is_ok):
                            for report in reports:
                                with log_service.step("create_error_report"):
                                    omnitron_integration.do_action(
                                        key="create_error_report", objects=report
                                    )

                        if response_data:
                            failed_reason_type = None
                        else:
                            failed_reason_type = FailedReasonType.remote.value

                        batch_request_object_dto = dict(
                            pk=cancellation_request.id,
                            version_date=cancellation_request.modified_date,
                            content_type=ContentType.cancellation_request.value,
                            failed_reason_type=failed_reason_type,
                            remote_id=remote_cancellation_request,
                        )

                        batch_request_object_list.append(batch_request_object_dto)

                    status = BatchRequestStatus.fail.value
                    if any(
                        [
                            br["failed_reason_type"] is None
                            for br in batch_request_object_list
                        ]
                    ):
                        status = BatchRequestStatus.done.value

                    omnitron_integration.batch_request.objects = (
                        batch_request_object_list
                    )
                    service = self.batch_service(omnitron_integration.channel_id)

                    if status == BatchRequestStatus.done.value:
                        with log_service.step("batch_to_done"):
                            service.to_done(
                                batch_request=omnitron_integration.batch_request
                            )
                    else:
                        with log_service.step("batch_to_fail"):
                            service.to_fail(
                                batch_request=omnitron_integration.batch_request
                            )
        except Exception as fatal:
            log_service.add_exception(fatal)
            raise
        finally:
            log_service.save()

    def get_channel_order_item(self, omnitron_integration, order_item):
        channel_id = omnitron_integration.channel_id
        end_point = ChannelIntegrationActionEndpoint(channel_id=channel_id)
        params = {
            "channel": channel_id,
            "content_type_name": ContentType.order_item.value,
            "object_id": order_item,
        }
        integration_action = end_point.list(params=params)
        if not integration_action:
            return Exception("Order item remote id not found: {}".format(order_item))
        if len(integration_action) > 1:
            return Exception(
                "Multiple order item remote id found: {}".format(order_item)
            )
        return integration_action[0].remote_id

    def get_channel_reason(self, omnitron_integration, omnitron_reason):
        configuration = omnitron_integration.channel.conf
        # configuration icinde yer alan reason_mapping anahtari bir json objesi,
        # key channel_reason value omnitron_reason olan bir dict objesinden
        # channel_reason'u döndürür
        for channel_reason, omni_reason in configuration.get("reason_mapping", {}):
            if omnitron_reason == omni_reason:
                return channel_reason

        return "10"

    def get_channel_cancellation_request(
        self, omnitron_integration, omnitron_cancel_request_pk
    ):
        channel_id = omnitron_integration.channel_id
        end_point = ChannelIntegrationActionEndpoint(channel_id=channel_id)
        params = {
            "channel": channel_id,
            "content_type_name": ContentType.cancellation_request.value,
            "object_id": omnitron_cancel_request_pk,
        }
        integration_action = end_point.list(params=params)
        if not integration_action:
            return Exception(
                "Cancellation request remote id not found: {}".format(
                    omnitron_cancel_request_pk
                )
            )
        if len(integration_action) > 1:
            return Exception(
                "Multiple cancellation request remote id found: {}".format(
                    omnitron_cancel_request_pk
                )
            )
        return integration_action[0].remote_id



from typing import List

from omnisdk.omnitron.models import BatchRequest, ProductImage

from channel_app.core import settings
from channel_app.core.data import BatchRequestResponseDto, ErrorReportDto
from channel_app.core.settings import OmnitronIntegration, ChannelIntegration
from channel_app.logs.services import LogService
from channel_app.omnitron.batch_request import ClientBatchRequest
from channel_app.omnitron.constants import ContentType


class ImageService(object):
    batch_service = ClientBatchRequest

    def update_product_images(
        self, is_sync=True, is_success_log=True, add_product_objects=False
    ):
        log_service = LogService()
        log_service.create_flow(name="Update Product Images")

        try:
            with log_service.step("update_product_images"):
                with OmnitronIntegration(
                    content_type=ContentType.product_image.value
                ) as omnitron_integration:
                    with log_service.step("get_updated_images"):
                        product_images = omnitron_integration.do_action(
                            key="get_updated_images"
                        )

                    first_product_image_count = len(product_images)
                    if add_product_objects:
                        with log_service.step("get_product_objects"):
                            product_images = (
                                product_images
                                and omnitron_integration.do_action(
                                    key="get_product_objects", objects=product_images
                                )
                            )

                    if not product_images:
                        if first_product_image_count:
                            omnitron_integration.batch_request.objects = None
                            with log_service.step("batch_to_fail"):
                                self.batch_service(
                                    omnitron_integration.channel_id
                                ).to_fail(omnitron_integration.batch_request)
                        return

                    product_images: List[ProductImage]

                    with log_service.step("send_updated_images"):
                        response_data, reports, data = ChannelIntegration().do_action(
                            key="send_updated_images",
                            objects=product_images,
                            batch_request=omnitron_integration.batch_request,
                            is_sync=is_sync,
                        )

                    # tips
                    response_data: List[BatchRequestResponseDto]
                    reports: List[ErrorReportDto]
                    data: List[ProductImage]

                    if not is_sync:
                        if reports[0].is_ok:
                            with log_service.step("batch_send_to_remote"):
                                self.batch_service(
                                    settings.OMNITRON_CHANNEL_ID
                                ).to_sent_to_remote(
                                    batch_request=omnitron_integration.batch_request
                                )
                        else:
                            is_sync = True

                    if reports and (is_success_log or not reports[0].is_ok):
                        for report in reports:
                            with log_service.step("create_error_report"):
                                omnitron_integration.do_action(
                                    key="create_error_report", objects=report
                                )

                    if is_sync:
                        with log_service.step("process_image_batch_requests"):
                            omnitron_integration.do_action(
                                key="process_image_batch_requests",
                                objects=response_data,
                            )
        except Exception as fatal:
            log_service.add_exception(fatal)
            raise
        finally:
            log_service.save()

    def insert_product_images(
        self, is_sync=True, is_success_log=True, add_product_objects=False
    ):
        log_service = LogService()
        log_service.create_flow(name="Insert Product Images")

        try:
            with log_service.step("insert_product_images"):
                with OmnitronIntegration(
                    content_type=ContentType.product_image.value
                ) as omnitron_integration:
                    with log_service.step("get_inserted_images"):
                        product_images = omnitron_integration.do_action(
                            key="get_inserted_images"
                        )

                    first_product_image_count = len(product_images)

                    if add_product_objects:
                        with log_service.step("get_product_objects"):
                            product_images = (
                                product_images
                                and omnitron_integration.do_action(
                                    key="get_product_objects", objects=product_images
                                )
                            )

                    if not product_images:
                        if first_product_image_count:
                            omnitron_integration.batch_request.objects = None
                            with log_service.step("batch_to_fail"):
                                self.batch_service(
                                    omnitron_integration.channel_id
                                ).to_fail(omnitron_integration.batch_request)
                        return

                    product_images: List[ProductImage]

                    with log_service.step("send_inserted_images"):
                        response_data, reports, data = ChannelIntegration().do_action(
                            key="send_inserted_images",
                            objects=product_images,
                            batch_request=omnitron_integration.batch_request,
                            is_sync=is_sync,
                        )

                    # tips
                    response_data: List[BatchRequestResponseDto]
                    reports: List[ErrorReportDto]
                    data: List[ProductImage]

                    if not is_sync:
                        if reports[0].is_ok:
                            with log_service.step("batch_send_to_remote"):
                                self.batch_service(
                                    settings.OMNITRON_CHANNEL_ID
                                ).to_sent_to_remote(
                                    batch_request=omnitron_integration.batch_request
                                )
                        else:
                            is_sync = True

                    if reports and (is_success_log or not reports[0].is_ok):
                        for report in reports:
                            with log_service.step("create_error_report"):
                                omnitron_integration.do_action(
                                    key="create_error_report", objects=report
                                )

                    if is_sync:
                        with log_service.step("process_image_batch_requests"):
                            omnitron_integration.do_action(
                                key="process_image_batch_requests",
                                objects=response_data,
                            )
        except Exception as fatal:
            log_service.add_exception(fatal)
            raise
        finally:
            log_service.save()

    def get_image_batch_requests(self, is_success_log=True):
        log_service = LogService()
        log_service.create_flow(name="Get Image Batch Requests")

        try:
            with log_service.step("get_image_batch_requests"):
                with OmnitronIntegration(create_batch=False) as omnitron_integration:
                    with log_service.step("get_batch_requests"):
                        batch_request_data = omnitron_integration.do_action(
                            "get_batch_requests",
                            params={
                                "status": ["sent_to_remote", "ongoing"],
                                "content_type": ContentType.product_image.value,
                            },
                        )
                    # tips
                    batch_request_data: List[BatchRequest]

                    for batch_request in batch_request_data:
                        with log_service.step("check_images"):
                            (
                                response_data,
                                reports,
                                data,
                            ) = ChannelIntegration().do_action(
                                key="check_images",
                                objects=batch_request,
                                batch_request=batch_request,
                            )

                        # tips
                        response_data: List[BatchRequestResponseDto]
                        reports: List[ErrorReportDto]
                        data: BatchRequest

                        if reports and (is_success_log or not reports[0].is_ok):
                            for report in reports:
                                with log_service.step("create_error_report"):
                                    omnitron_integration.do_action(
                                        key="create_error_report", objects=report
                                    )

                        if response_data:
                            omnitron_integration.batch_request = batch_request
                            with log_service.step("process_image_batch_requests"):
                                omnitron_integration.do_action(
                                    key="process_image_batch_requests",
                                    objects=response_data,
                                )
        except Exception as fatal:
            log_service.add_exception(fatal)
            raise
        finally:
            log_service.save()



from typing import List

from channel_app.core import settings
from channel_app.core.data import CategoryTreeDto, ErrorReportDto, AttributeDto
from channel_app.core.settings import OmnitronIntegration, ChannelIntegration
from channel_app.logs.services import LogService
from channel_app.omnitron.constants import ContentType


class SetupService(object):
    def create_or_update_category_tree_and_nodes(self, is_success_log=False):
        log_service = LogService()
        log_service.create_flow(name="(Setup) Create or Update Category Tree and Nodes")

        try:
            with OmnitronIntegration(
                content_type=ContentType.category_tree.value) as omnitron_integration:
                channel_integration = ChannelIntegration()
                with log_service.step("get_category_tree_and_nodes"):
                    category_tree, report, _ = channel_integration.do_action(
                        key='get_category_tree_and_nodes',
                        batch_request=omnitron_integration.batch_request)

                category_tree: CategoryTreeDto
                report: ErrorReportDto

                if report and (is_success_log or not report.is_ok):
                    with log_service.step("create_error_report"):
                        omnitron_integration.do_action(
                            key='create_error_report',
                            objects=report)

                with log_service.step("create_or_update_category_tree_and_nodes"):
                    omnitron_integration.do_action(
                        key='create_or_update_category_tree_and_nodes',
                        objects=category_tree)
        except Exception as fatal:
            log_service.add_exception(fatal)
            raise
        finally:
            log_service.save()

    def create_or_update_category_attributes(self, is_success_log=False):
        log_service = LogService()
        log_service.create_flow(name="Create or Update Category Attributes")

        try:
            with OmnitronIntegration(
                content_type=ContentType.attribute.value) as omnitron_integration:
                channel_integration = ChannelIntegration()
                with log_service.step("get_category_ids"):
                    category_integration_actions = omnitron_integration.do_action(
                        key='get_category_ids')

                for category_ia in category_integration_actions:
                    if not category_ia.remote_id:
                        continue
                    
                    with log_service.step("get_category_attributes"):
                        category, report, data = channel_integration.do_action(
                            key='get_category_attributes',
                            objects=category_ia,
                            batch_request=omnitron_integration.batch_request
                        )
                    if report and (is_success_log or not report.is_ok):
                        with log_service.step("create_error_report"):
                            omnitron_integration.do_action(
                                key='create_error_report',
                                objects=report)

                    category = category if category.attributes else None
                    with log_service.step("create_or_update_category_attributes"):
                        omnitron_integration.do_action(
                            key='create_or_update_category_attributes',
                            objects=(category_ia, category))
        except Exception as fatal:
            log_service.add_exception(fatal)
            raise
        finally:
            log_service.save()

    def create_or_update_attributes(self, is_success_log=False):
        log_service = LogService()
        log_service.create_flow(name="Create or Update Attributes")

        try:
            with OmnitronIntegration(
                content_type=ContentType.attribute.value) as omnitron_integration:
                channel_integration = ChannelIntegration()
                with log_service.step("get_attributes"):
                    attributes, report, data = channel_integration.do_action(
                        key='get_attributes',
                        batch_request=omnitron_integration.batch_request
                    )

                attributes: List[AttributeDto]
                reports: ErrorReportDto

                if report and (is_success_log or not report.is_ok):
                    with log_service.step("create_error_report"):
                        omnitron_integration.do_action(
                            key='create_error_report',
                            objects=report)

                for attribute in attributes:
                    with log_service.step("create_or_update_channel_attribute"):
                        attr = omnitron_integration.do_action(
                            key="create_or_update_channel_attribute",
                            objects={"name": attribute.name,
                                    "remote_id": attribute.remote_id},
                        )[0]
                        
                    with log_service.step("get_or_create_channel_attribute_schema"):
                        omnitron_integration.do_action(
                            key="get_or_create_channel_attribute_schema",
                            objects={"name": f"{settings.OMNITRON_CHANNEL_ID} {attribute.name} Schema"},
                        )
                    for attr_value in attribute.values:
                        with log_service.step("create_or_update_channel_attribute_value", metadata={
                            "attribute_name": attr_value.name,
                            "remote_id": attr_value.remote_id
                        }):
                            omnitron_integration.do_action(
                                key="create_or_update_channel_attribute_value",
                                objects={
                                    "attribute": attr.pk,
                                    "label": attr_value.name,
                                    "value": attr_value.remote_id,
                                    "remote_id": attr_value.remote_id})
        except Exception as fatal:
            log_service.add_exception(fatal)
            raise
        finally:
            log_service.save()

    def update_channel_conf_schema(self):
        log_service = LogService()
        log_service.create_flow(name="Update Channel Conf Schema")

        try:
            with OmnitronIntegration(
                content_type=ContentType.channel.value) as omnitron_integration:
                channel_integration = ChannelIntegration()
                with log_service.step("get_channel_conf_schema"):
                    schema, _, _ = channel_integration.do_action(
                        key='get_channel_conf_schema',
                        batch_request=omnitron_integration.batch_request)

                with log_service.step("update_channel_conf_schema"):
                    omnitron_integration.do_action(key="update_channel_conf_schema",
                                                objects=schema)
        except Exception as fatal:
            log_service.add_exception(fatal)
            raise
        finally:
            log_service.save()



from typing import List

from omnisdk.omnitron.models import ProductPrice, BatchRequest

from channel_app.core import settings
from channel_app.core.data import BatchRequestResponseDto, ErrorReportDto
from channel_app.core.settings import OmnitronIntegration, ChannelIntegration
from channel_app.logs.services import LogService
from channel_app.omnitron.batch_request import ClientBatchRequest
from channel_app.omnitron.constants import ContentType


class PriceService(object):
    batch_service = ClientBatchRequest

    def update_product_prices(
        self,
        is_sync=True,
        is_success_log=True,
        add_product_objects=False,
        add_stock=False,
    ):
        log_service = LogService()
        log_service.create_flow(name="Update Product Prices")

        try:
            with log_service.step("update_product_prices"):
                with OmnitronIntegration(
                    content_type=ContentType.product_price.value
                ) as omnitron_integration:
                    with log_service.step("get_updated_prices"):
                        product_prices = omnitron_integration.do_action(
                            key="get_updated_prices"
                        )

                    first_product_price_count = len(product_prices)

                    if add_product_objects:
                        with log_service.step("get_product_objects"):
                            product_prices = (
                                product_prices
                                and omnitron_integration.do_action(
                                    key="get_product_objects", objects=product_prices
                                )
                            )

                    if add_stock:
                        with log_service.step("get_stocks_from_product_prices"):
                            product_prices = (
                                product_prices
                                and omnitron_integration.do_action(
                                    key="get_stocks_from_product_prices",
                                    objects=product_prices,
                                    stock_list=omnitron_integration.catalog.stock_list,
                                )
                            )

                    if not product_prices:
                        if first_product_price_count:
                            omnitron_integration.batch_request.objects = None
                            with log_service.step("batch_to_fail"):
                                self.batch_service(
                                    omnitron_integration.channel_id
                                ).to_fail(omnitron_integration.batch_request)
                        return

                    product_prices: List[ProductPrice]

                    with log_service.step("send_updated_prices"):
                        response_data, reports, data = ChannelIntegration().do_action(
                            key="send_updated_prices",
                            objects=product_prices,
                            batch_request=omnitron_integration.batch_request,
                            is_sync=is_sync,
                        )

                    # tips
                    response_data: List[BatchRequestResponseDto]
                    reports: List[ErrorReportDto]
                    data: List[ProductPrice]

                    if not is_sync:
                        if reports[0].is_ok:
                            with log_service.step("batch_send_to_remote"):
                                self.batch_service(
                                    settings.OMNITRON_CHANNEL_ID
                                ).to_sent_to_remote(
                                    batch_request=omnitron_integration.batch_request
                                )
                        else:
                            is_sync = True

                    if reports and (is_success_log or not reports[0].is_ok):
                        for report in reports:
                            with log_service.step("create_error_report"):
                                omnitron_integration.do_action(
                                    key="create_error_report", objects=report
                                )

                    if is_sync:
                        with log_service.step("process_price_batch_requests"):
                            omnitron_integration.do_action(
                                key="process_price_batch_requests",
                                objects=response_data,
                            )
        except Exception as fatal:
            log_service.add_exception(fatal)
            raise
        finally:
            log_service.save()

    def insert_product_prices(
        self,
        is_sync=True,
        is_success_log=True,
        add_product_objects=False,
        add_stock=False,
    ):
        log_service = LogService()
        log_service.create_flow(name="Insert Product Prices")

        try:
            with log_service.step("insert_product_prices"):
                with OmnitronIntegration(
                    content_type=ContentType.product_price.value
                ) as omnitron_integration:
                    with log_service.step("get_inserted_prices"):
                        product_prices = omnitron_integration.do_action(
                            key="get_inserted_prices"
                        )

                    first_product_price_count = len(product_prices)

                    if add_product_objects:
                        with log_service.step("get_product_objects"):
                            product_prices = (
                                product_prices
                                and omnitron_integration.do_action(
                                    key="get_product_objects", objects=product_prices
                                )
                            )

                    if add_stock:
                        with log_service.step("get_stocks_from_product_prices"):
                            product_prices = (
                                product_prices
                                and omnitron_integration.do_action(
                                    key="get_stocks_from_product_prices",
                                    objects=product_prices,
                                    stock_list=omnitron_integration.catalog.stock_list,
                                )
                            )

                    if not product_prices:
                        if first_product_price_count:
                            omnitron_integration.batch_request.objects = None
                            with log_service.step("batch_to_fail"):
                                self.batch_service(
                                    omnitron_integration.channel_id
                                ).to_fail(omnitron_integration.batch_request)
                        return

                    product_prices: List[ProductPrice]

                    with log_service.step("send_inserted_prices"):
                        response_data, reports, data = ChannelIntegration().do_action(
                            key="send_inserted_prices",
                            objects=product_prices,
                            batch_request=omnitron_integration.batch_request,
                            is_sync=is_sync,
                        )

                    # tips
                    response_data: List[BatchRequestResponseDto]
                    reports: List[ErrorReportDto]
                    data: List[ProductPrice]

                    if not is_sync:
                        if reports[0].is_ok:
                            with log_service.step("batch_send_to_remote"):
                                self.batch_service(
                                    settings.OMNITRON_CHANNEL_ID
                                ).to_sent_to_remote(
                                    batch_request=omnitron_integration.batch_request
                                )
                        else:
                            is_sync = True

                    if reports and (is_success_log or not reports[0].is_ok):
                        for report in reports:
                            with log_service.step("create_error_report"):
                                omnitron_integration.do_action(
                                    key="create_error_report", objects=report
                                )

                    if is_sync:
                        with log_service.step("process_price_batch_requests"):
                            omnitron_integration.do_action(
                                key="process_price_batch_requests",
                                objects=response_data,
                            )
        except Exception as fatal:
            log_service.add_exception(fatal)
            raise
        finally:
            log_service.save()

    def insert_product_prices_from_extra_price_list(
        self,
        is_sync=True,
        is_success_log=True,
        add_stock=False,
        add_product_objects=False,
    ):
        log_service = LogService()
        log_service.create_flow(name="Insert Product Prices from Extra Price List")

        try:
            with log_service.step("insert_product_prices_from_extra_price_list"):
                with log_service.step("get_currency_mappings"):
                    currency_mappings = self.get_currency_mappings()
                
                for price_list_id, country_code in currency_mappings.items():
                    with OmnitronIntegration(
                        content_type=ContentType.product_price.value
                    ) as omnitron_integration:
                        with log_service.step("get_inserted_prices_from_extra_price_list"):
                            product_prices = omnitron_integration.do_action(
                                key="get_inserted_prices_from_extra_price_list",
                                objects=price_list_id,
                            )
                        first_product_price_count = len(product_prices)

                        if add_product_objects:
                            with log_service.step("get_product_objects"):
                                product_prices = product_prices and omnitron_integration.do_action(
                                    key="get_product_objects", objects=product_prices
                                )

                        if add_stock:
                            with log_service.step("get_stocks_from_product_prices"):
                                product_prices = product_prices and omnitron_integration.do_action(
                                    key="get_stocks_from_product_prices",
                                    objects=product_prices,
                                    stock_list=omnitron_integration.catalog.stock_list,
                                )

                        product_prices: List[ProductPrice]
                        if product_prices:
                            with log_service.step("send_inserted_prices"):
                                response_data, reports, data = ChannelIntegration().do_action(
                                    key="send_inserted_prices",
                                    objects=(product_prices, country_code),
                                    batch_request=omnitron_integration.batch_request,
                                    is_sync=is_sync,
                                )

                            # tips
                            response_data: List[BatchRequestResponseDto]
                            reports: List[ErrorReportDto]
                            data: List[ProductPrice]

                            if not is_sync:
                                if reports[0].is_ok:
                                    with log_service.step("batch_send_to_remote"):
                                        self.batch_service(
                                            settings.OMNITRON_CHANNEL_ID
                                        ).to_sent_to_remote(
                                            batch_request=omnitron_integration.batch_request
                                        )
                                else:
                                    is_sync = True

                            if reports and (is_success_log or not reports[0].is_ok):
                                for report in reports:
                                    with log_service.step("create_error_report"):
                                        omnitron_integration.do_action(
                                            key="create_error_report", objects=report
                                        )

                            if is_sync:
                                with log_service.step("process_price_batch_requests"):
                                    omnitron_integration.do_action(
                                        key="process_price_batch_requests", objects=response_data
                                    )
                        else:
                            if first_product_price_count:
                                omnitron_integration.batch_request.objects = None
                                with log_service.step("batch_to_fail"):
                                    self.batch_service(omnitron_integration.channel_id).to_fail(
                                        omnitron_integration.batch_request
                                    )
        except Exception as fatal:
            log_service.add_exception(fatal)
            raise
        finally:
            log_service.save()

    def update_product_prices_from_extra_price_list(
        self,
        is_sync=True,
        is_success_log=True,
        add_stock=False,
        add_product_objects=False,
    ):
        log_service = LogService()
        log_service.create_flow(name="Update Product Prices from Extra Price List")

        try:
            with log_service.step("get_currency_mappings"):
                currency_mappings = self.get_currency_mappings()

            for price_list_id, country_code in currency_mappings.items():
                with OmnitronIntegration(
                    content_type=ContentType.product_price.value
                ) as omnitron_integration:
                    with log_service.step("get_updated_prices_from_extra_price_list"):
                        product_prices = omnitron_integration.do_action(
                            key="get_updated_prices_from_extra_price_list",
                            objects=price_list_id,
                        )
                    first_product_price_count = len(product_prices)
                    if add_product_objects:
                        with log_service.step("get_product_objects"):
                            product_prices = product_prices and omnitron_integration.do_action(
                                key="get_product_objects", objects=product_prices
                            )

                    if add_stock:
                        with log_service.step("get_stocks_from_product_prices"):
                            product_prices = product_prices and omnitron_integration.do_action(
                                key="get_stocks_from_product_prices",
                                objects=product_prices,
                                stock_list=omnitron_integration.catalog.stock_list,
                            )

                    product_prices: List[ProductPrice]
                    if product_prices:
                        with log_service.step("send_updated_prices"):
                            response_data, reports, data = ChannelIntegration().do_action(
                                key="send_updated_prices",
                                objects=(product_prices, country_code),
                                batch_request=omnitron_integration.batch_request,
                                is_sync=is_sync,
                            )

                        # tips
                        response_data: List[BatchRequestResponseDto]
                        reports: List[ErrorReportDto]
                        data: List[ProductPrice]

                        if not is_sync:
                            if reports[0].is_ok:
                                with log_service.step("batch_send_to_remote"):
                                    self.batch_service(
                                        settings.OMNITRON_CHANNEL_ID
                                    ).to_sent_to_remote(
                                        batch_request=omnitron_integration.batch_request
                                    )
                            else:
                                is_sync = True

                        if reports and (is_success_log or not reports[0].is_ok):
                            for report in reports:
                                with log_service.step("create_error_report"):
                                    omnitron_integration.do_action(
                                        key="create_error_report", objects=report
                                    )

                        if is_sync:
                            with log_service.step("process_price_batch_requests"):
                                omnitron_integration.do_action(
                                    key="process_price_batch_requests", objects=response_data
                                )
                    else:
                        if first_product_price_count:
                            omnitron_integration.batch_request.objects = None
                            with log_service.step("batch_to_fail"):
                                self.batch_service(omnitron_integration.channel_id).to_fail(
                                    omnitron_integration.batch_request
                                )
        except Exception as fatal:
            log_service.add_exception(fatal)
            raise
        finally:
            log_service.save()

    def get_price_batch_requests(self, is_success_log=True):
        log_service = LogService()
        log_service.create_flow(name="Get Price Batch Requests")

        try:
            with OmnitronIntegration(create_batch=False) as omnitron_integration:
                with log_service.step("get_batch_requests"):
                    batch_request_data = omnitron_integration.do_action(
                        "get_batch_requests",
                        params={
                            "status": ["sent_to_remote", "ongoing"],
                            "content_type": ContentType.product_price.value,
                        },
                    )
                # tips
                batch_request_data: List[BatchRequest]

                for batch_request in batch_request_data:
                    with log_service.step("check_prices"):
                        response_data, reports, data = ChannelIntegration().do_action(
                            key="check_prices",
                            objects=batch_request,
                            batch_request=batch_request,
                        )

                    # tips
                    response_data: List[BatchRequestResponseDto]
                    reports: List[ErrorReportDto]
                    data: BatchRequest

                    if reports and (is_success_log or not reports[0].is_ok):
                        for report in reports:
                            with log_service.step("create_error_report"):
                                omnitron_integration.do_action(
                                    key="create_error_report", objects=report
                                )

                    if response_data:
                        omnitron_integration.batch_request = batch_request
                        with log_service.step("process_price_batch_requests"):
                            omnitron_integration.do_action(
                                key="process_price_batch_requests", objects=response_data
                            )
        except Exception as fatal:
            log_service.add_exception(fatal)
            raise
        finally:
            log_service.save()

    def get_currency_mappings(self):
        """
        Process is like below:
         We get key value list of the currency mappings, filter Omnitron
         with key field
        {
            "1": "ae",
            "2": "us"
        }
        :return:
        """
        with OmnitronIntegration(create_batch=False) as integration:
            currency_mappings = integration.channel.conf.get("CURRENCY_MAPPINGS", {})
            price_lists = integration.catalog.extra_price_lists
            if integration.catalog.price_list:
                price_lists.append(integration.catalog.price_list)
            filtered_currency_mappings = {
                price_list: currency_mappings[price_list]
                for price_list in map(str, price_lists)
                if price_list in currency_mappings
            }
        return filtered_currency_mappings



from typing import List

from omnisdk.omnitron.models import (
    Product,
    IntegrationAction,
    BatchRequest,
)

from channel_app.core import settings
from channel_app.core.data import ProductBatchRequestResponseDto, ErrorReportDto
from channel_app.core.settings import OmnitronIntegration, ChannelIntegration
from channel_app.logs.services import LogService
from channel_app.omnitron.batch_request import ClientBatchRequest
from channel_app.omnitron.constants import ContentType


class ProductService(object):
    batch_service = ClientBatchRequest

    def insert_products(
        self,
        add_mapped=True,
        add_stock=True,
        add_price=True,
        add_categories=True,
        is_sync=True,
        is_success_log=True,
    ):
        log_service = LogService()
        log_service.create_flow(name="Insert Products")

        try:
            with log_service.step("insert_products"):
                with OmnitronIntegration(
                    content_type=ContentType.product.value
                ) as omnitron_integration:
                    with log_service.step("get_inserted_products"):
                        products = omnitron_integration.do_action(
                            key="get_inserted_products"
                        )

                    first_product_count = len(products)

                    if add_mapped:
                        with log_service.step("get_mapped_products"):
                            products = products and omnitron_integration.do_action(
                                key="get_mapped_products", objects=products
                            )

                    if add_stock:
                        with log_service.step("get_product_stocks"):
                            products = products and omnitron_integration.do_action(
                                key="get_product_stocks", objects=products
                            )

                    if add_price:
                        with log_service.step("get_product_prices"):
                            products = products and omnitron_integration.do_action(
                                key="get_product_prices", objects=products
                            )

                    if add_categories:
                        with log_service.step("get_product_categories"):
                            products = products and omnitron_integration.do_action(
                                key="get_product_categories", objects=products
                            )

                    if not products:
                        if first_product_count:
                            omnitron_integration.batch_request.objects = None

                            with log_service.step("batch_to_fail"):
                                self.batch_service(
                                    omnitron_integration.channel_id
                                ).to_fail(omnitron_integration.batch_request)

                        return

                    products: List[Product]

                    with log_service.step("send_inserted_products"):
                        response_data, reports, data = ChannelIntegration().do_action(
                            key="send_inserted_products",
                            objects=products,
                            batch_request=omnitron_integration.batch_request,
                            is_sync=is_sync,
                        )

                    # tips
                    response_data: List[ProductBatchRequestResponseDto]
                    reports: List[ErrorReportDto]
                    data: List[Product]

                    if not is_sync:
                        if reports[0].is_ok:

                            with log_service.step("batch_send_to_remote"):
                                self.batch_service(
                                    settings.OMNITRON_CHANNEL_ID
                                ).to_sent_to_remote(
                                    batch_request=omnitron_integration.batch_request
                                )
                        else:
                            is_sync = True

                    if reports and (is_success_log or not reports[0].is_ok):
                        for report in reports:
                            with log_service.step("create_error_report"):
                                omnitron_integration.do_action(
                                    key="create_error_report", objects=report
                                )

                    if is_sync:
                        with log_service.step("process_product_batch_requests"):
                            omnitron_integration.do_action(
                                key="process_product_batch_requests",
                                objects=response_data,
                            )
        except Exception as fatal:
            log_service.add_exception(fatal)
            raise
        finally:
            log_service.save()

    def update_products(
        self,
        add_mapped=True,
        add_stock=True,
        add_price=True,
        add_categories=True,
        is_sync=True,
        is_success_log=True,
    ):
        log_service = LogService()
        log_service.create_flow(name="Update Products")

        try:
            with log_service.step("update_products"):
                with OmnitronIntegration(
                    content_type=ContentType.product.value
                ) as omnitron_integration:
                    with log_service.step("get_updated_products"):
                        products = omnitron_integration.do_action(
                            key="get_updated_products"
                        )

                    first_product_count = len(products)

                    if add_mapped:
                        with log_service.step("get_mapped_products"):
                            products = products and omnitron_integration.do_action(
                                key="get_mapped_products", objects=products
                            )

                    if add_stock:
                        with log_service.step("get_product_stocks"):
                            products = products and omnitron_integration.do_action(
                                key="get_product_stocks", objects=products
                            )

                    if add_price:
                        with log_service.step("get_product_prices"):
                            products = products and omnitron_integration.do_action(
                                key="get_product_prices", objects=products
                            )

                    if add_categories:
                        with log_service.step("get_product_categories"):
                            products = products and omnitron_integration.do_action(
                                key="get_product_categories", objects=products
                            )

                    if not products:
                        if first_product_count:
                            omnitron_integration.batch_request.objects = None
                            with log_service.step("batch_to_fail"):
                                self.batch_service(
                                    omnitron_integration.channel_id
                                ).to_fail(omnitron_integration.batch_request)

                        return

                    products: List[Product]

                    with log_service.step("send_updated_products"):
                        response_data, reports, data = ChannelIntegration().do_action(
                            key="send_updated_products",
                            objects=products,
                            batch_request=omnitron_integration.batch_request,
                            is_sync=is_sync,
                        )

                    # tips
                    response_data: List[ProductBatchRequestResponseDto]
                    reports: List[ErrorReportDto]
                    data: List[Product]

                    if not is_sync:
                        if reports[0].is_ok:
                            with log_service.step("batch_send_to_remote"):
                                self.batch_service(
                                    settings.OMNITRON_CHANNEL_ID
                                ).to_sent_to_remote(
                                    batch_request=omnitron_integration.batch_request
                                )
                        else:
                            is_sync = True

                    if reports and (is_success_log or not reports[0].is_ok):
                        for report in reports:
                            with log_service.step("create_error_report"):
                                omnitron_integration.do_action(
                                    key="create_error_report", objects=report
                                )

                    if is_sync:
                        with log_service.step("process_product_batch_requests"):
                            omnitron_integration.do_action(
                                key="process_product_batch_requests",
                                objects=response_data,
                            )
        except Exception as fatal:
            log_service.add_exception(fatal)
            raise
        finally:
            log_service.save()

    def delete_products(
        self, is_sync=True, is_content_object=True, is_success_log=True
    ):
        log_service = LogService()
        log_service.create_flow("Delete Products")

        try:
            with log_service.step("delete_products"):
                with OmnitronIntegration(
                    content_type=ContentType.integration_action.value
                ) as omnitron_integration:
                    with log_service.step("get_deleted_products"):
                        products_integration_action = omnitron_integration.do_action(
                            key="get_deleted_products"
                        )
                    if not products_integration_action:
                        return

                    with log_service.step("get_content_objects_from_integrations"):
                        products_integration_action = omnitron_integration.do_action(
                            key="get_content_objects_from_integrations",
                            objects=products_integration_action,
                        )
                    products_integration_action: List[IntegrationAction]

                    with log_service.step("send_deleted_products"):
                        response_data, reports, data = ChannelIntegration().do_action(
                            key="send_deleted_products",
                            objects=products_integration_action,
                            batch_request=omnitron_integration.batch_request,
                            is_sync=is_sync,
                        )

                    # tips
                    response_data: List[ProductBatchRequestResponseDto]
                    reports: List[ErrorReportDto]
                    data: List[IntegrationAction]

                    if not is_sync:
                        if reports[0].is_ok:
                            with log_service.step("batch_to_commit"):
                                self.batch_service(
                                    settings.OMNITRON_CHANNEL_ID
                                ).to_commit(
                                    batch_request=omnitron_integration.batch_request
                                )
                            with log_service.step("batch_send_to_remote"):
                                self.batch_service(
                                    settings.OMNITRON_CHANNEL_ID
                                ).to_sent_to_remote(
                                    batch_request=omnitron_integration.batch_request
                                )
                        else:
                            is_sync = True

                    if reports and (is_success_log or not reports[0].is_ok):
                        for report in reports:
                            with log_service.step("create_error_report"):
                                omnitron_integration.do_action(
                                    key="create_error_report", objects=report
                                )

                    if is_sync:
                        with log_service.step("process_delete_product_batch_requests"):
                            omnitron_integration.do_action(
                                key="process_delete_product_batch_requests",
                                objects=response_data,
                            )
        except Exception as fatal:
            log_service.add_exception(fatal)
            raise
        finally:
            log_service.save()

    def get_delete_product_batch_requests(self, is_success_log=True):
        log_service = LogService()
        log_service.create_flow(name="Get Delete Product Batch Requests")

        try:
            with log_service.step("get_delete_product_batch_requests"):
                with OmnitronIntegration(create_batch=False) as omnitron_integration:
                    with log_service.step("get_batch_requests"):
                        batch_request_data = omnitron_integration.do_action(
                            "get_batch_requests",
                            params={
                                "status": ["sent_to_remote", "ongoing"],
                                "content_type": ContentType.integration_action.value,
                            },
                        )
                    # tips
                    batch_request_data: List[BatchRequest]

                    for batch_request in batch_request_data:
                        with log_service.step("check_deleted_products"):
                            (
                                response_data,
                                report,
                                data,
                            ) = ChannelIntegration().do_action(
                                key="check_deleted_products",
                                objects=batch_request,
                                batch_request=batch_request,
                            )

                        # tips
                        response_data: List[ProductBatchRequestResponseDto]
                        report: ErrorReportDto
                        data: BatchRequest

                        if report and (is_success_log or not report.is_ok):
                            with log_service.step("create_error_report"):
                                omnitron_integration.do_action(
                                    key="create_error_report", objects=report
                                )
                        if response_data:
                            omnitron_integration.batch_request = batch_request
                            with log_service.step(
                                "process_delete_product_batch_requests"
                            ):
                                omnitron_integration.do_action(
                                    key="process_delete_product_batch_requests",
                                    objects=response_data,
                                )
        except Exception as fatal:
            log_service.add_exception(fatal)
        finally:
            log_service.save()

    def get_product_batch_requests(self, is_success_log=True):
        log_service = LogService()
        log_service.create_flow(name="Get Product Batch Requests")

        try:
            with log_service.step("get_product_batch_requests"):
                with OmnitronIntegration(create_batch=False) as omnitron_integration:
                    with log_service.step("get_batch_requests"):
                        batch_request_data = omnitron_integration.do_action(
                            "get_batch_requests",
                            params={
                                "status": ["sent_to_remote", "ongoing"],
                                "content_type": ContentType.product.value,
                            },
                        )
                    # tips
                    batch_request_data: List[BatchRequest]

                    for batch_request in batch_request_data:
                        with log_service.step("check_products"):
                            (
                                response_data,
                                report,
                                data,
                            ) = ChannelIntegration().do_action(
                                key="check_products",
                                objects=batch_request,
                                batch_request=batch_request,
                            )

                        # tips
                        response_data: List[ProductBatchRequestResponseDto]
                        report: ErrorReportDto
                        data: BatchRequest

                        if report and (is_success_log or not report.is_ok):
                            with log_service.step("create_error_report"):
                                omnitron_integration.do_action(
                                    key="create_error_report", objects=report
                                )

                        if response_data:
                            omnitron_integration.batch_request = batch_request
                            with log_service.step("process_product_batch_requests"):
                                omnitron_integration.do_action(
                                    key="process_product_batch_requests",
                                    objects=response_data,
                                )
        except Exception as fatal:
            log_service.add_exception(fatal)
            raise
        finally:
            log_service.save()



from decimal import Decimal
from typing import List
from channel_app.core.settings import OmnitronIntegration
from channel_app.logs.services import LogService
from channel_app.omnitron.integration import OmnitronIntegration as IOmnitronIntegration
from channel_app.omnitron.constants import ContentType
from omnisdk.omnitron.models import Product, ProductStock


class ReportService:
    @staticmethod
    def get_not_for_sale_products(limit: int = 10) -> List[Product]:
        """
        Get not for sale products from Omnitron API.
        """

        log_service = LogService()
        log_service.create_flow(name="Get Not For Sale Products")

        try:
            with OmnitronIntegration(
                content_type=ContentType.product.value,
            ) as omnitron_integration:
                omnitron_integration: IOmnitronIntegration

                with log_service.step("get_not_for_sale_products"):
                    products: List[Product] = omnitron_integration.do_action(
                        key="get_inserted_products",
                        params={
                            "catalogitem__isnull": True,
                        },
                        objects={
                            "limit": limit,
                        },
                    )

                return products
        except Exception as fatal:
            log_service.add_exception(fatal)
            raise
        finally:
            log_service.save()

    @staticmethod
    def get_price_differences_from_products(marketplace_products: List[Product]):
        """
        Get price differences from products.
        :param marketplace_products: List of products from marketplace
        :return: List of products with price differences

        marketplace_products: List[Product] = [
            Product(
                pk=1,
                price=100.0,
                sku="test-sku-1",
                ...
            ),
            Product(
                pk=3,
                price=200.0,
                sku="test-sku-2",
                ...
            ),
        ]
        """

        log_service = LogService()
        log_service.create_flow(name="Get Price Differences")

        try:
            with OmnitronIntegration(
                content_type=ContentType.product_price.value,
            ) as omnitron_integration:
                omnitron_integration: IOmnitronIntegration

                with log_service.step("get_product_prices"):
                    omnitron_products = omnitron_integration.do_action(
                        key="get_product_prices",
                        objects=marketplace_products,
                    )

                with log_service.step("get_price_differences"):
                    differences = []
                    for omnitron_product in omnitron_products:
                        if omnitron_product.productprice is None:
                            continue

                        if omnitron_product.productprice.price is None:
                            continue

                        for marketplace_product in marketplace_products:
                            if omnitron_product.sku == marketplace_product.sku:
                                if Decimal(
                                    omnitron_product.productprice.price
                                ) != Decimal(marketplace_product.price):
                                    differences.append(
                                        {
                                            "product_pk": omnitron_product.pk,
                                            "product_sku": omnitron_product.sku,
                                            "price_difference": abs(
                                                Decimal(
                                                    omnitron_product.productprice.price
                                                )
                                                - Decimal(marketplace_product.price)
                                            ),
                                            "updated_at": omnitron_product.productprice.modified_date,
                                        }
                                    )

                    return differences
        except Exception as fatal:
            log_service.add_exception(fatal)
            raise
        finally:
            log_service.save()

    @staticmethod
    def get_stock_differences_from_products(marketplace_products: List[Product]):
        """
        Get stock differences from products.
        :param marketplace_products: List of products from marketplace
        :return: List of products with stock differences

        marketplace_products: List[Product] = [
            Product(
                pk=1,
                stock=100,
                sku="test-sku-1",
                ...
            ),
            Product(
                pk=3,
                stock=200,
                sku="test-sku-2",
                ...
            ),
        ]
        """
        log_service = LogService()
        log_service.create_flow(name="Get Stock Differences")

        try:
            with OmnitronIntegration(
                content_type=ContentType.product_stock.value,
            ) as omnitron_integration:
                omnitron_integration: IOmnitronIntegration

                with log_service.step("get_product_stocks"):
                    omnitron_products = omnitron_integration.do_action(
                        key="get_product_stocks",
                        objects=marketplace_products,
                    )

                with log_service.step("get_stock_differences"):
                    differences = []
                    for omnitron_product in omnitron_products:
                        if omnitron_product.productstock is None:
                            continue

                        if omnitron_product.productstock.stock is None:
                            continue

                        for marketplace_product in marketplace_products:
                            if omnitron_product.sku == marketplace_product.sku:
                                if int(omnitron_product.productstock.stock) != int(
                                    marketplace_product.stock
                                ):
                                    differences.append(
                                        {
                                            "product_pk": omnitron_product.pk,
                                            "product_sku": omnitron_product.sku,
                                            "stock_difference": abs(
                                                int(omnitron_product.productstock.stock)
                                                - int(marketplace_product.stock)
                                            ),
                                            "updated_at": omnitron_product.productstock.modified_date,
                                        }
                                    )
                    return differences
        except Exception as fatal:
            log_service.add_exception(fatal)
            raise
        finally:
            log_service.save()


class DashboardService:
    @staticmethod
    def get_total_sku_from_catalog(limit: int = 10) -> List[Product]:
        """
        Get total SKU from catalog.
        """
        
        log_service = LogService()
        log_service.create_flow(name="Get Total SKU from Catalog")

        try:
            with OmnitronIntegration(
                content_type=ContentType.product.value,
            ) as omnitron_integration:
                omnitron_integration: IOmnitronIntegration

                with log_service.step("get_total_sku_from_catalog"):
                    products: List[Product] = omnitron_integration.do_action(
                        key="get_inserted_products",
                        objects={
                            "limit": limit,
                        },
                    )

                return products
        except Exception as fatal:
            log_service.add_exception(fatal)
            raise
        finally:
            log_service.save()

    @staticmethod
    def get_total_sku_on_sale_from_catalog(limit: int = 10) -> List[Product]:
        """
        Get total SKU on sale from catalog.
        """

        log_service = LogService()
        log_service.create_flow(name="Get Total SKU on Sale from Catalog")

        try:
            with OmnitronIntegration(
                content_type=ContentType.product.value,
            ) as omnitron_integration:
                omnitron_integration: IOmnitronIntegration

                with log_service.step("get_total_sku_on_sale_from_catalog"):
                    products: List[Product] = omnitron_integration.do_action(
                        key="get_inserted_products",
                        params={
                            "catalogitem__isnull": False,
                        },
                        objects={
                            "limit": limit,
                        },
                    )

                return products
        except Exception as fatal:
            log_service.add_exception(fatal)
            raise
        finally:
            log_service.save()

    @staticmethod
    def get_product_stocks_from_catalog(products: List[Product]) -> List[ProductStock]:
        """
        Get product stocks from catalog.
        """
        log_service = LogService()
        log_service.create_flow(name="Get Product Stocks from Catalog")

        try:
            with OmnitronIntegration(
                content_type=ContentType.product_stock.value,
            ) as omnitron_integration:
                omnitron_integration: IOmnitronIntegration

                with log_service.step("get_product_stocks_from_catalog"):
                    stocks = omnitron_integration.do_action(
                        key="get_product_stocks",
                        objects=products,
                    )
                    
                return stocks
        except Exception as fatal:
            log_service.add_exception(fatal)
            raise
        finally:
            log_service.save()

    @staticmethod
    def get_total_stock_from_product_stocks(product_stocks: List[ProductStock]) -> int:
        """
        Get total stock from catalog.
        """
        log_service = LogService()
        log_service.create_flow(name="Get Total Stock from Catalog")
        
        try:
            total_stock = 0
            for stock in product_stocks:
                total_stock += stock.productstock.stock if stock.productstock else 0

            return total_stock
        except Exception as fatal:
            log_service.add_exception(fatal)
            raise
        finally:
            log_service.save()


import os
import sys
from alembic.config import Config
from alembic import command

def migrate():
    base_dir = os.path.dirname(os.path.abspath(__file__))
    ini_path = os.path.join(base_dir, 'alembic.ini')

    alembic_cfg = Config(ini_path)
    alembic_cfg.set_main_option("script_location", os.path.join(base_dir, "database", "migrations"))

    command.upgrade(alembic_cfg, "head")


if __name__ == "__main__":
    try:
        migrate()
    except Exception as e:
        print(f"Migration failed: {e}")

        sys.exit(1)


from sqlalchemy import create_engine
from channel_app.core import settings


class DatabaseService:
    def create_engine(self):
        DATABASE_URL = f"postgresql+psycopg2://{settings.DB_USER}:{settings.DB_PASSWORD}@{settings.DB_HOST}:{settings.DB_PORT}/{settings.DB_NAME}"
        engine = create_engine(DATABASE_URL, echo=False)
        return engine


from datetime import datetime, timezone
import uuid
from sqlalchemy import JSON, Column, DateTime, ForeignKey, Integer, String, Enum as SqlEnum, Text
from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.orm import DeclarativeBase, relationship

from channel_app.logs.enums import LogFlowAuthor, LogStepStatus


class Base(DeclarativeBase):
    pass


class LogFlowModel(Base):
    __tablename__ = "log_flows"

    id = Column(UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    transaction_id = Column(UUID(as_uuid=True), unique=True, nullable=False)
    flow_name = Column(String(255), nullable=False)
    flow_author = Column(SqlEnum(LogFlowAuthor), default=LogFlowAuthor.system, nullable=False)

    started_at = Column(DateTime(timezone=True), nullable=False, default=lambda: datetime.now(timezone.utc))
    ended_at = Column(DateTime(timezone=True), nullable=True)

    status = Column(SqlEnum(LogStepStatus), nullable=True)
    s3_key = Column(Text, nullable=True)

    def __repr__(self):
        return f"<FlowLog(transaction_id={self.transaction_id}, flow_name={self.flow_name})>"
    

class LogStepModel(Base):
    __tablename__ = "log_steps"

    id = Column(UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    flow_id = Column(UUID(as_uuid=True), ForeignKey("log_flows.id", ondelete="CASCADE"), nullable=False)
    step_name = Column(String(255), nullable=False)
    status = Column(SqlEnum(LogStepStatus, native_enum=False), nullable=False)
    start_time = Column(DateTime(timezone=True), nullable=False)
    end_time = Column(DateTime(timezone=True))
    duration_ms = Column(Integer)
    error_message = Column(String)
    step_metadata = Column(JSON)

    exceptions = relationship("LogStepExceptionModel", back_populates="step", cascade="all, delete-orphan")
    

class LogStepExceptionModel(Base):
    __tablename__ = "log_step_exceptions"

    id = Column(UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    step_id = Column(UUID(as_uuid=True), ForeignKey("log_steps.id", ondelete="CASCADE"), nullable=False)
    type = Column(String(128), nullable=False)
    message = Column(String)
    traceback = Column(String)
    created_at = Column(DateTime(timezone=True), nullable=False)

    step = relationship("LogStepModel", back_populates="exceptions")
